#include "playlistaspectalbumgroupmodel.h"
#include "debug.h"

enum Roles {
    PlayListIDRole = Qt::UserRole + 1,
    KeyRole = Qt::UserRole + 2,
    FilenameRole = Qt::UserRole + 3,
    ArtistRole = Qt::UserRole + 4,
    AlbumRole = Qt::UserRole + 5
};

PlayListAspectAlbumGroupModel::PlayListAspectAlbumGroupModel(PlayListAspectAlbumGroupBuilder *aspect)
    :	QAbstractListModel(aspect),
        aspect_(aspect)
{
    DGUARDMETHOD;
    QHash<int, QByteArray> names;
    names[PlayListIDRole] = "playListID";
    names[KeyRole] = "key";
    names[FilenameRole] = "filename";
    names[ArtistRole] = "artist";
    names[AlbumRole] = "album";
    setRoleNames(names);
}

int PlayListAspectAlbumGroupModel::rowCount(const QModelIndex &parent) const
{
    return aspect_->groups().count();
}

int PlayListAspectAlbumGroupModel::columnCount(const QModelIndex &parent) const
{
    return 0;
}

QVariant PlayListAspectAlbumGroupModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    GroupItemList &itemList = aspect_->groups();

    if (index.row() >= itemList.size())
        return QVariant();

    GroupItem *groupItem = itemList.at(index.row());

    switch (role)
    {
    case PlayListIDRole:
        return (qulonglong)groupItem->playlistID;
        break;
    case KeyRole:
        return groupItem->key;
        break;
    case FilenameRole:
        return groupItem->filename;
        break;
    case ArtistRole:
        return groupItem->artist;
        break;
    case AlbumRole:
        return groupItem->album;
        break;
    default:
        return QVariant();
    }
}

QVariant PlayListAspectAlbumGroupModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    return QVariant();
}

void PlayListAspectAlbumGroupModel::sort(int column, Qt::SortOrder order)
{
    DGUARDMETHOD;
    /*
    if (column != aspect_->sortedColumn())
        aspect_->setSortParams(column, order == Qt::AscendingOrder);
    else
        aspect_->setSortParams(aspect_->sortedColumn(), !aspect_->isSortAscending());
    */
    DPRINTF("Header section %d clicked. Preparing playlist item index...", column);
}

void PlayListAspectAlbumGroupModel::rereadData()
{
    emit layoutChanged();
}
